#ifndef TPC_GEM_H
#define TPC_GEM_H
#include "Polya.h"
#include "TF2.h"
//-----------------------------------------------------------------------------
// ==========
// class GEM 
// ==========
// (Description)
//    Class to describe a GEM module.
// (Provides)
//    TPC::GEM               : GEM object
// (Update Record)
//    2015/07/11  K.Fujii     Original version.
//-----------------------------------------------------------------------------
namespace TPC {
class GEM : public TF2 {
public:
    GEM(double sgprf =  0.27, // PRF width      [mm]
        double wpad  =  1.25, // pad pitch      [mm]
        double lpad  =  5.20, // pad row pitch  [mm]
        double vgap  = 50.00, // drift velocity [um/ns]
        double igap  =  2.00, // induction gap  [mm]
	double gain  = 2000., // average gas gain
        double theta =  0.50, // polya parameter
        bool   isodd = true); // (true,false)=(odd row, even row)

    inline double GetSigmaPRF () const { return GetParameter(0); }
    inline double GetPadWidth () const { return GetParameter(1); }
    inline double GetPadLength() const { return GetParameter(2); }
    inline double GetVdriftGap() const { return fVdrift; }
    inline double GetGapLength() const { return fIndGap; }
    inline bool   IsOdd       () const { return fIsOdd;  }
    inline double GetOffset   () const { return fIsOdd ? GetPadWidth()/2. : 0.; }

    inline void   SetIsOdd(bool isodd) { fIsOdd = isodd; }

    double Prf    (const double *x, const double *p);
    double Fx     (int a, double x, double y);

    double GasGain();

    double Integral(double xlo, double xhi, double ylo, double yhi, double eps = 1.e-8);

private:
    double fVdrift;  // drift velocity in the induction gap [um/ns]
    double fIndGap;  // induction gap [mm]
    double fGain;    // average gas gain
    Polya  fPolya;   // Polya object for gas gain fluctuation
    bool   fIsOdd;   // (true,false)=(odd row, even row)

    ClassDef(GEM, 1)   // GEM module class
};
}
#endif
