// script to plot a single pulse shape from PASA.
// You should run this in compile mode
//   $ root -l
//   [0] .L Pulse.C+
//   [1] Pulse();
//
#include "Pasa.h"
#include "TGraph.h"
#include "TLine.h"

using namespace std;

TF1 *Pulse1(bool ig = true, double norm = 1., double tpk0 = 120., int n0 = 4)
{ 
  //--
  // Draw PASA output pulse for various peaking times.
  //--
  const double w0    = 50.; // time bin width [ns]
  const double noise = 50.; // noise counts
  TPC::Pasa  pca(tpk0,n0,w0,noise,norm);

  // Wrapper function class to work around a strange cling problem.
  class Func {
  public:
	  Func(TPC::Pasa *pcap) : _pcap(pcap) {}

	  double Evaluate(double *xv, double *pv) {
		  return _pcap->Pulse(xv,pv);
	  }
  private:
	  TPC::Pasa *_pcap;
  };

  Func fn(&pca);

  double xmin = -200.;
  double xmax = +500.;
  int    npar = 5;
  //TF1 *fp = new TF1("pca",&pca,&TPC::Pasa::Pulse, 
  //                  xmin, xmax, npar, "TPC::Pasa","Pulse");
  TF1 *fp = new TF1("fn", &fn, &Func::Evaluate, xmin, xmax, npar);
  fp->SetMinimum(0.);
  fp->SetMaximum(0.03);
  fp->SetParameter(0,tpk0);
  fp->SetParameter(1,n0);
  fp->SetParameter(4,norm);

  if (ig) fp->DrawCopy();
  return fp;
}
